@@ -35,6 +35,7 @@
 #include "src/iwd.h"
 #include "src/module.h"
 #include "src/dbus.h"
+#include "src/netconfig.h"
 #include "src/resolve.h"
 
 struct resolve_ops {
@@ -131,11 +132,9 @@ static void systemd_link_generic_reply(struct l_dbus_message *message,
 	const char *name;
 	const char *text;
 
-	if (!l_dbus_message_is_error(message))
+	if (!l_dbus_message_get_error(message, &name, &text))
 		return;
 
-	l_dbus_message_get_error(message, &name, &text);
-
 	l_error("resolve-systemd: Failed to modify the %s entries. %s: %s",
 							type, name, text);
 }
@@ -559,6 +558,38 @@ static const struct resolve_method_ops resolve_method_resolvconf_ops = {
 	.alloc = resolve_resolvconf_alloc,
 };
 
+static void resolve_none_destroy(struct resolve *resolve)
+{
+	l_free(resolve);
+}
+
+static struct resolve_ops none_ops = {
+	.destroy = resolve_none_destroy,
+};
+
+static int resolve_none_init(void)
+{
+	return 0;
+}
+
+static void resolve_none_exit(void)
+{
+}
+
+static struct resolve *resolve_none_alloc(uint32_t ifindex)
+{
+	struct resolve *r = l_new(struct resolve, 1);
+
+	_resolve_init(r, ifindex, &none_ops);
+	return r;
+}
+
+static const struct resolve_method_ops resolve_method_none_ops = {
+	.init = resolve_none_init,
+	.exit = resolve_none_exit,
+	.alloc = resolve_none_alloc,
+};
+
 static const struct resolve_method_ops *configured_method;
 
 struct resolve *resolve_new(uint32_t ifindex)
@@ -575,37 +606,22 @@ static const struct {
 } resolve_method_ops_list[] = {
 	{ "systemd", &resolve_method_systemd_ops },
 	{ "resolvconf", &resolve_method_resolvconf_ops },
+	{ "none", &resolve_method_none_ops },
 	{ }
 };
 
 static int resolve_init(void)
 {
 	const char *method_name;
-	bool enabled;
 	uint8_t i;
 
-	if (!l_settings_get_bool(iwd_get_config(), "General",
-					"EnableNetworkConfiguration",
-					&enabled)) {
-		if (!l_settings_get_bool(iwd_get_config(), "General",
-					"enable_network_config", &enabled))
-			enabled = false;
-	}
-
-	if (!enabled)
+	if (!netconfig_enabled())
 		return 0;
 
 	method_name = l_settings_get_value(iwd_get_config(), "Network",
 						"NameResolvingService");
-	if (!method_name) {
-		method_name = l_settings_get_value(iwd_get_config(), "General",
-							"dns_resolve_method");
-		if (method_name)
-			l_warn("[General].dns_resolve_method is deprecated, "
-				"use [Network].NameResolvingService");
-		else /* Default to systemd-resolved service. */
-			method_name = "systemd";
-	}
+	if (!method_name)
+		method_name = "systemd";
 
 	for (i = 0; resolve_method_ops_list[i].name; i++) {
 		if (strcmp(resolve_method_ops_list[i].name, method_name))
