@@ -133,7 +133,10 @@ struct l_settings *network_lookup_security(const char *network)
 	l_debug("Loading %s", path);
 
 	conf = l_settings_new();
-	l_settings_load_from_file(conf, path);
+	if (!l_settings_load_from_file(conf, path)) {
+		l_settings_free(conf);
+		conf = NULL;
+	}
 
 	l_free(path);
 
@@ -161,6 +164,7 @@ static void network_storage_watch_cb(const char *filename,
 		network_reload(name);
 		break;
 	case L_DIR_WATCH_EVENT_ACCESSED:
+	case L_DIR_WATCH_EVENT_ATTRIB:
 		break;
 	}
 
